#include "resoff.h"
#include "checks.h"
#include "mask.h"

/*======================================================================
  Function cancel
  Input : filename of database, argument number and arguments of main
          function
  Output : update database and error messages
  cancel a reservation from database file
======================================================================*/
int cancel (char *filename, int argc, char **argv) {

    extern int fd;                   /* file descriptor of database file */
    int desk_number;                 /* desk number */
    int db_index;                    /* database block index */
    int loc;                         /* location of a file */
    int len;                         /* number of bytes written */
    Room slot;                       /* a time block */
    sigset_t newset, oldset;         /* new and old signal sets */

    db_index = get_db_index (argv);  /* from arguments, get index of block */
    if ( db_index < 0 ) {            /* if error         */
        remove_lock();               /* remove lock file */
        exit (ERROR);                /* and exit         */
    }
    loc = db_index * sizeof(slot);   /* calculate location of a block */

    CHECK (fd = open (filename, O_CREAT | O_RDWR,
		      S_IRUSR | S_IWUSR));

    if ( empty_db (fd) ) {
        printf("Empty database, please reserve first\n");
	CHECK (unlink (filename));
	remove_lock();
	exit (ERROR);
    }

    SIGFILL(newset);
    BLOCK(newset, oldset);
    CHECK (lseek (fd, loc, SEEK_SET));
    CHECK (len = read (fd, &slot, sizeof(slot))); /* read a block to update */
    UNBLOCK(newset, oldset);

    desk_number = get_desk_number(slot, argc, argv);

    /* update a block to be empty */
    if ( desk_number < 4 ) {
        slot.count--;
	slot.empty[desk_number] = TRUE;
	BLOCK(newset, oldset);
	CHECK (lseek (fd, loc, SEEK_SET));
	CHECK (len = write (fd, &slot, sizeof(slot)));
	BLOCK(newset, oldset);
    }

    CHECK (close (fd));
    
    return (NORMAL);

}

/*======================================================================
  Function get_desk_number
  Input : a time block, argument number and arguments of main function
  Output : return matched desk number, error messages
  find a user matched desk or check validity of input cancel options
======================================================================*/
int get_desk_number(Room slot, int argc, char **argv) {

    int i;                      /* loop index */
    extern uid_t uid;           /* user id */
    extern uid_t euid;          /* effective user id */
    uid_t target_uid;           /* target user id */
    struct passwd *puid;        /* passwd for user name and uid */

    if ( argc == 4 ) {          /* without optional username */
        target_uid = uid;
        for (i=0; i<4; i++) {
	    if ( !slot.empty[i] ) {
	        if ( target_uid == slot.desk[i].uid ) {
		    return (i);
		}
	    }
	}
	printf("There is no reservation to cancel for you\n");
    }
    else if ( argc == 5 ) {     /* username given to delete */
	puid = getpwnam(argv[4]);
	if ( puid == NULL ) {   /* there is no such user name */
	    printf("No \"%s\" username exist\n", argv[4]);
	    remove_lock();
	    exit(ERROR);
	}
	target_uid = puid->pw_uid;
	/* if user id is owner of database or user name is the user's */
        /* then search given user's desk                              */
	if ( uid == euid || target_uid == uid ) {
	    for (i=0; i<4; i++) {
	        if ( !slot.empty[i] ) {
	            if ( target_uid == slot.desk[i].uid ) {
		        return (i);
		    }
		}
	    }
	    fprintf(stderr, "No %s\'s reservation at %s on %s\n",
		    argv[4], argv[3], argv[2]);
	}
	else {
	    fprintf(stderr, 
		   "You are not allowed to cancel other user's reservation\n");
	}
    }

    return (9999);                    /* large enough not to match any desk */

}
