#include "config.h"
#include "resoff.h"

Sigfunc *
signal(int signo, Sigfunc *func)
{
    struct sigaction  act, oact;
    
    act.sa_handler = func;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    if (signo == SIGALRM) {
#ifdef  SA_INTERRUPT
        act.sa_flags |= SA_INTERRUPT;   /* SunOS 4.x */
#endif
    } else {
#ifdef  SA_RESTART
        act.sa_flags |= SA_RESTART;     /* SVR4, 44BSD */
#endif
    }
    if (sigaction(signo, &act, &oact) < 0)
        return(SIG_ERR);
    return(oact.sa_handler);
}


Sigfunc *
Signal(int signo, Sigfunc *func)        /* for our signal() function */
{
    Sigfunc *sigfunc;
    
    if ( (sigfunc = signal(signo, func)) == SIG_ERR) {
        fprintf(stderr, "signal error");
        exit(0);
    }
    
    return(sigfunc);
}

void
connect_alarm(int signo)
{
    fprintf(stderr, "connection error\n");
    exit(0);
}

int main(int argc, char **argv) {

    char *server;
    struct hostent *hosts;
    char *strport;
    int portnumber;
    int sockfd;
    struct sockaddr_in servaddr;
    Sigfunc *sigfunc;
    
    if ( !(server = getenv("RESOFFHOST")) ) {
        server = "mom.cs.fsu.edu";          /**/ /*mom*/
    }
    if ( !(hosts = gethostbyname(server)) ) {
        fprintf(stderr, "host\n");
        exit(0);
    }

    if ( !(strport = getenv("RESOFFPORT")) ) {
        portnumber = SERV_PORT;
    }
    else {
        portnumber = atoi(strport);
    }
    
    if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0 ){
        fprintf(stderr, "socket\n");
        exit(0);
    }

    memset (&servaddr, 0, sizeof(servaddr));
    servaddr.sin_family = AF_INET;
    servaddr.sin_port = htons(portnumber);
    servaddr.sin_addr = *(struct in_addr *) (hosts->h_addr_list[0]);
    
    sigfunc = Signal(SIGALRM, connect_alarm);
    if (alarm(30) != 0) {
        fprintf(stderr, "Alarm was already set\n");
        exit(0);
    }

    if ( (connect(sockfd, (SA *) &servaddr, sizeof(servaddr))) < 0 ) {
        for(;;);
        fprintf(stderr, "connection fail\n");
        close(sockfd);
        if (errno == EINTR)
            errno = ETIMEDOUT;
    }
    printf("done\n");
    alarm(0);
    Signal(SIGALRM, sigfunc);

    exit(0);
}
