#include "udplib.h"
#include "addrinfo.h"

int udp_connect(const char *host, const char *serv)
{
    int sockfd, n;
    struct addrinfo hints, *res, *ressave;
    bzero(&hints, sizeof(struct addrinfo));
    hints.ai_family = AF_UNSPEC;
    hints.ai_socktype = SOCK_DGRAM;
    
    if ( (n = getaddrinfo(host, serv, &hints, &res)) != 0 ) {
        fprintf(stderr, "udp_connect error for %s, %s: ",
                host, serv);
        perror(errno);
        exit(-1);
    }
    
    ressave = res;
    
    do {
        sockfd = socket(res->ai_family, res->ai_socktype,
                        res->ai_protocol);
        if (sockfd < 0)
            continue;
        
        if (connect(sockfd, res->ai_addr, res->ai_addrlen) == 0)
            break;
        close (sockfd);
    } while ( (res = res->ai_next) != NULL);
    
    if (res == NULL) {
        fprintf(stderr, "udp_connect error for %s, %s", host, serv);
        exit(-1);
    }
    
    freeaddrinfo(ressave);
    
    return(sockfd);
}

int Udp_connect(const char *host, const char *serv)
{
    int n;
    
    if ( (n = udp_connect(host, serv)) < 0) {
        fprintf (stderr, "udp_connect error for %s, %s: ",
                 host, serv);
        perror(errno);
        exit(-1);
    }
    return(n);
}


