#include "xresoff.h"

/*======================================================================
  Function xlist
  Input : output list from resoffc program result
  Output : a resoff data list window
  make a list window and get the list of resoff database
======================================================================*/
void xlist ()
{
    int n;                        /* number of read character */
    int fd[2];                    /* file descriptors */
    pid_t pid;                    /* child pid */
    char line[1024];              /* buffer for read from resoffc */
    GtkWidget *window;
    GtkWidget *box1;
    GtkWidget *box2;
    GtkWidget *button;
    GtkWidget *separator;
    GtkWidget *table;
    GtkWidget *vscrollbar;
    GtkWidget *text;
    GdkColormap *cmap;
    GdkColor color;
    GdkFont *fixed_font;

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_usize (window, 610, 500);
    gtk_window_set_policy (GTK_WINDOW(window), TRUE, TRUE, FALSE);  

    gtk_window_set_title (GTK_WINDOW (window), "Xresoff List");
    gtk_container_set_border_width (GTK_CONTAINER (window), 0);    
  
    box1 = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), box1);
    gtk_widget_show (box1); 
  
    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_start (GTK_BOX (box1), box2, TRUE, TRUE, 0);
    gtk_widget_show (box2);
  
    table = gtk_table_new (2, 2, FALSE);
    gtk_table_set_row_spacing (GTK_TABLE (table), 0, 2);
    gtk_table_set_col_spacing (GTK_TABLE (table), 0, 2);
    gtk_box_pack_start (GTK_BOX (box2), table, TRUE, TRUE, 0);
    gtk_widget_show (table);
  
    /* Create the GtkText widget */
    text = gtk_text_new (NULL, NULL);
    gtk_text_set_editable (GTK_TEXT (text), FALSE);
    gtk_table_attach (GTK_TABLE (table), text, 0, 1, 0, 1,
                      GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                      GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
    gtk_widget_show (text);

    /* Add a vertical scrollbar to the GtkText widget */
    vscrollbar = gtk_vscrollbar_new (GTK_TEXT (text)->vadj);
    gtk_table_attach (GTK_TABLE (table), vscrollbar, 1, 2, 0, 1,
                      GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
    gtk_widget_show (vscrollbar);

    /* Get the system color map and allocate the color red */
    cmap = gdk_colormap_get_system();
    color.red = 0xffff;
    color.green = 0;
    color.blue = 0;
    if (!gdk_color_alloc(cmap, &color)) {
        g_error("couldn't allocate color");
    }

    /* Load a fixed font */
    fixed_font = gdk_font_load ("-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*");

    /* Realizing a widget creates a window for it,
     * ready for us to insert some text */
    gtk_widget_realize (text);

    /* Freeze the text widget, ready for multiple updates */
    gtk_text_freeze (GTK_TEXT (text));
  
    if (pipe(fd) < 0) {
        fprintf(stderr, "pipe error");
        exit(-1);
    }
    
    if ( (pid = fork()) < 0 ) {
        fprintf(stderr, "fork error");
        exit(-1);
    }
    else if (pid==0) {
        close(fd[0]);
        dup2(fd[1], 1);
        dup2(fd[1], 2);
        execl(pathname, "resoffc", "-l", NULL);
    }
    else {
        close(fd[1]);
        while ( (n = read(fd[0], line, 1024)) ){
            gtk_text_insert (GTK_TEXT (text), fixed_font, NULL,
                             NULL, line, n);
        }
    }

    separator = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);
    gtk_widget_show (separator);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);
    gtk_widget_show (box2);
  
    button = gtk_button_new_with_label ("close");
    gtk_signal_connect_object (GTK_OBJECT(button), "clicked",
                               GTK_SIGNAL_FUNC(gtk_widget_destroy),
                               GTK_OBJECT(window));

    gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (button);
    gtk_widget_show (button);

    gtk_widget_show (window);

}


