/***********************************************************************
  GUI main function of Assignment #6 of Advanced Unix Programming.
  Input : menu, days, times, course number, user name (GUI input)
          list of current reserved desk database through TCP/IP using
          client 'resoffc', result of the functions, error codes
  Output : GUI displays

  Author : Jungkee (Jake) Kim
  Date : December 4, 2000 (Due Date)
************************************************************************/
#include "xresoff.h"

static GtkItemFactoryEntry menu_items[] = {
  { "/_Resoff",       NULL,         NULL, 0, "<Branch>" },
  { "/Resoff/Reserve","<control>R", xresoff_menu, 1, NULL },
  { "/Resoff/Cancel", "<control>C", xresoff_menu, 2, NULL },
  { "/Resoff/List",   "<control>L", xresoff_menu, 3, NULL },
  { "/Resoff/sep1",   NULL,         NULL, 0, "<Separator>" },
  { "/Resoff/Quit",   "<control>Q", gtk_main_quit, 0, NULL },
  { "/_Options",      NULL,         NULL, 0, "<Branch>" },
  { "/Options/Host",  NULL,         option_menu, 1, NULL },
  { "/Options/Port",  NULL,         option_menu, 2, NULL },
  { "/_Help",         "<control>H", NULL, 0, "<LastBranch>" },
  { "/_Help/Help",    NULL,         help_show,  0, NULL },
  { "/_Help/About",   NULL,         help_about, 0, NULL },
};

int main( int argc,
          char *argv[] )
{
    GtkWidget *window;
    GtkWidget *main_vbox;
    GtkWidget *menubar;
    GtkWidget *vbox;
    GtkWidget *hbox;
    GtkWidget *label;
    GtkWidget *frame;
    char *a_server;                 /* environment server name */
    char *a_port;                   /* environment port number */
  
    if ( !(a_server = getenv("RESOFFHOST")) ) {
        sprintf(server, "default");
    }
    else {
        strcpy(server, a_server);
    }
    
    if ( !(a_port = getenv("RESOFFPORT")) ) {
        sprintf(port, "default");
    }
    else {
        strcpy(port, a_port);
    }
    
    get_env();
    gtk_init (&argc, &argv);
  
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_signal_connect (GTK_OBJECT (window), "destroy", 
                        GTK_SIGNAL_FUNC (gtk_main_quit), 
                        "WM destroy");
    gtk_window_set_title (GTK_WINDOW(window), "Xresoff");
    gtk_widget_set_usize (GTK_WIDGET(window), 350, 200);
  
    main_vbox = gtk_vbox_new (FALSE, 1);
    gtk_container_border_width (GTK_CONTAINER (main_vbox), 1);
    gtk_container_add (GTK_CONTAINER (window), main_vbox);
    gtk_widget_show (main_vbox);

    get_main_menu (window, &menubar);
    gtk_box_pack_start (GTK_BOX (main_vbox), menubar, FALSE, TRUE, 0);
    gtk_widget_show (menubar);

    /* Environment frame */
    frame = gtk_frame_new ("Environment");
    vbox = gtk_vbox_new(TRUE, 2);
    hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start (GTK_BOX (main_vbox), frame, FALSE, FALSE, 0);    
    gtk_container_add( GTK_CONTAINER (frame), vbox);
    gtk_container_add( GTK_CONTAINER (vbox), hbox);
    
    label = gtk_label_new("Server:");
    gtk_label_set_justify( GTK_LABEL (label), GTK_JUSTIFY_LEFT );
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 5);

    s_label = gtk_label_new((gchar *) server);
    gtk_box_pack_start (GTK_BOX (hbox), s_label, FALSE, FALSE, 0);
    
    hbox = gtk_hbox_new(FALSE, 0);
    gtk_container_add( GTK_CONTAINER (vbox), hbox);
    
    label = gtk_label_new("Port:");
    gtk_label_set_justify( GTK_LABEL (label), GTK_JUSTIFY_LEFT );
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 5);
    p_label = gtk_label_new((gchar *) port);
    gtk_box_pack_start (GTK_BOX (hbox), p_label, FALSE, FALSE, 0);
    
    m_label = gtk_label_new("");
    gtk_box_pack_end (GTK_BOX (main_vbox), m_label, TRUE, TRUE, 2);
    gtk_label_set_text(GTK_LABEL(m_label), "xresoff\nmade by Jungkee Kim");
  
    gtk_widget_show_all (window);
    gtk_main ();
    
    return(0);
}

/*======================================================================
  Function get_main_menu
  Input : GtkWidget window and menubar
          itemfactory (global variable)
  Output : main menubar
  make a main menubar from itemfactory
======================================================================*/
void get_main_menu( GtkWidget  *window,
                    GtkWidget **menubar)
{
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);

  accel_group = gtk_accel_group_new ();

  /* This function initializes the item factory.
     Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
              or GTK_TYPE_OPTION_MENU.
     Param 2: The path of the menu.
     Param 3: A pointer to a gtk_accel_group.  The item factory sets up
              the accelerator table while generating menus.
  */

  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", 
				       accel_group);

  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

  /* Attach the new accelerator group to the window. */
  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  if (menubar)
    /* Finally, return the actual menu bar created by the item factory. */ 
    *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}
