#include "resoff.h"
#include "checks.h"
#include "mask.h"

/*======================================================================
  Function list
  Input : database file name
  Output : list of reserved desks
           heading, day, time, user name, course name
  print the contents of the database
======================================================================*/
int list (char *filename) {

    int i, j;                  /* loop index */
    extern int fd;             /* file descriptor of database file */
    extern char buf[7][1200];  /* buffer for printing day data  */
                               /* becase actual data is not     */
                               /* orderd in MTWRFSX but MWFTRSX */
    char str[128];             /* string for user name and course name */
    int len;                   /* number of bytes read */
    int loc;                   /* block location */
    int day;                   /* numbered day */
    Room slot;                 /* a time block */
    struct passwd *user;       /* passwd struct for user name */
    sigset_t newset, oldset;   /* new and old signal sets */

    CHECK (fd = open (filename, O_CREAT | O_RDWR,
		      S_IRUSR | S_IWUSR));

    if ( empty_db (fd) ) {
        printf("Empty database, please reserve first\n");
	CHECK (unlink (filename));
	remove_lock();
	exit (ERROR);
    }
    
    print_head();              /* printing header of reservation table */

    SIGFILL(newset);

    for (i=0; i<69; i++) {
        day = print_day(i);     /* print a day and return day value */
	loc = i * sizeof(slot); /* block location */
	BLOCK(newset, oldset);
	CHECK (lseek(fd, loc, SEEK_SET));
        CHECK (len = read (fd, &slot, sizeof(slot)));
	UNBLOCK(newset, oldset);
	if ( slot.count > 0 ) {
	    time_print(day, i);
	    for (j=0; j<4; j++) {
	        if ( !slot.empty[j] ) {
		    user = getpwuid(slot.desk[j].uid); /* get user name */
		    sprintf (str, " %s %s", user->pw_name,
			    slot.desk[j].class);
		    strcat (buf[day], str);
		}
	    }
	}
    }

    printf("%s\n\n%s\n\n%s\n\n%s\n\n%s\n\n%s\n\n%s\n",
	   buf[0], buf[3], buf[1], buf[4], buf[2], buf[5], buf[6]);
    

    CHECK (close (fd));

    return(0);

}

void print_head() {

    printf("--------------------------------------------------------------\n");
    printf("                     Office Reservation\n");
    printf("--------------------------------------------------------------\n");

}

/*======================================================================
  Function print_day
  Input : index of a time block in the database file
          MWF : 11 time blocks a day  TRSX : 9 time blocks a day
  Output : print days on correspond buffer array
           return a numbered value of a day ( eg. Monday 0 Tuesday 3 )
  print to buffer a day name and return numbered value of a day
======================================================================*/
int print_day(int i) {

    int day;                             /* numbered value of a day */
    extern char buf[7][1200];            /* temporary buffer to print */

    switch(i) {
    case 0:                              /* starting point of Monday */
        sprintf(buf[0], "Monday  ");
	return (0);
        break;
    case 11:                             /* starting point of Wednesday */
        sprintf(buf[1], "Wednesday");
	return (1);
        break;
    case 22:
        sprintf(buf[2], "Friday  ");
	return (2);
        break;
    case 33:                             /* starting point of Tuesday */
        sprintf(buf[3], "Tuesday ");
	return (3);
        break;
    case 42:
        sprintf(buf[4], "Thursday");
	return (4);
	break;
    case 51:
        sprintf(buf[5], "Saturday");
	return (5);
        break;
    case 60:
        sprintf(buf[6], "Sunday ");
	return (6);
        break;
    default:
      day = get_day(i);           /* calculate corresponded day value */
	return (day);
        break;
    }

    return(ERROR);

}

/*======================================================================
  Function get_day
  Input : index of a time block in the database file
  Output : return a numbered value of a day ( eg. Monday 0 Tuesday 3 )
  return a numbered value corresponded block index of the database file
======================================================================*/
int get_day(int i) {

    if ( i < 11 ) { return (0); }
    else if ( i < 22 ) { return (1); }
    else if ( i < 33 ) { return (2); }
    else if ( i < 42 ) { return (3); }
    else if ( i < 51 ) { return (4); }
    else if ( i < 60 ) { return (5); }
    else { return (6); }
    
    return (ERROR);

}

/*======================================================================
  Function time_print
  Input : day value, index of a time block in the database file
  Output : string buffer filled with corresponded time
  from day and block index, print a time string corresponding index
======================================================================*/
int time_print(int day, int i) {
    
    int index;                  /* index of time of each day */
    char str[128];              /* temporary index to save time print */
    extern char buf[7][1200];   /* temporary buffer to print each day */
    char mwf_day[11][6] = {     /* Monday, Wednesday, Friday times */
        "08:00", "09:05", "10:10", "11:15", "12:20",
	"13:25", "14:30", "15:35", "17:15", "18:45", "20:15" };
    char trsx_day[9][6] = {     /* the other days times */
        "08:00", "09:30", "11:00", "12:30",
	"14:00", "15:35", "17:15", "18:45", "20:15" };

    switch (day) {
    case 0:
    case 1:
    case 2:
        index = i % 11;         /* MWF has 11 time blocks */
	sprintf (str, "\n%s ", mwf_day[index]);
	strcat (buf[day], str);
	break;
    case 3:
    case 4:
    case 5:
    case 6:
        index = ( i - 33 ) % 9; /* TRSX has 9 time blocks after MWF blocks */
	sprintf(str, "\n%s ", trsx_day[index]);
	strcat (buf[day], str);
	break;
    default:
        break;
    }

    return (0);

}






