#define _POSIX_C_SOURCE 199506L

#include <ctype.h>
#include "resoff.h"
#include "checks.h"
#include "mask.h"

/*======================================================================
  Function reserve
  Input : database filename, arguments of main function
  Output : time block data saved in database file
  reserving a desk on a time slot
======================================================================*/
int reserve (char *filename, char **argv) {

    int i;
    int db_index;                  /* index of a time block */
    int loc;                       /* location of a random access file */
    extern int fd;                 /* file description of database file */
    extern uid_t uid;              /* user id */
    int len;                       /* length of a string */
    Room slot;                     /* a time block */
    sigset_t newset, oldset;       /* old and new signal masks */

    course_check (argv[4]);        /* error check of course name */

    db_index = get_db_index(argv); /* get corresponding index in database */
    if ( db_index < 0 ) {          /* if failed */
        remove_lock();             /* remove lock file */
        exit(ERROR);               /* ERROR defined -1 */
    }
    loc = db_index * sizeof(slot); /* location of corresponding block */
        
    CHECK (fd = open (filename, O_CREAT | O_RDWR,
		      S_IRUSR | S_IWUSR));

    if ( empty_db (fd) ) {        /* if no database file, make and initilize */
	initdb(filename);
    }

    SIGFILL(newset);              /* setup full signal mask */
    BLOCK(newset, oldset);        /* block system interrupt */

    CHECK (lseek (fd, loc, SEEK_SET));
    CHECK (len = read (fd, &slot, sizeof(slot)));

    UNBLOCK(newset, oldset);      /* unblock system interrupt */

    /* check empty desk and validity of input, and update data */
    if ( slot.count > -1 && slot.count < 4 ) {  /* count is 0, 1, 2, 3 */
        slot.count++;
	make_upper(argv[4]);                    /* upper case of course name */
	validity(db_index, slot, argv[4]);      /* input validity check */
        for (i=0; i<4; i++) {
	    if ( slot.empty[i] ) {              /* if a desk is empty */
	        slot.empty[i] = FALSE;
	        slot.desk[i].uid = uid;
		make_upper(argv[4]);
		strcpy(slot.desk[i].class, argv[4]);
		BLOCK(newset, oldset);
		CHECK (lseek (fd, loc, SEEK_SET));
		CHECK (len = write (fd, &slot, sizeof(slot)));
		CHECK (close(fd));
		UNBLOCK(newset, oldset);
		return (NORMAL);
	    }
	}
    }
    else if ( slot.count == 4 ) {                /* desks are full */
        fprintf(stderr, "No empty desk at %s on %s\n", argv[3], argv[2]);
	CHECK (close(fd));
	remove_lock();
	exit (ERROR);
    }
    else {	                                 /* unreachable count number */
	fprintf(stderr, "Slot count error in reserve.\n");
	CHECK (close(fd));
	remove_lock();
	exit (ERROR);
    }
    
    return(NORMAL);

}

/*======================================================================
  Function get_db_index
  Input : arguments of main function
  Output : corresponding index of database file
  get a corresoponded index of data block
======================================================================*/
int get_db_index (char **argv) {

    int i, j;
    int index;
    char mwf_day[11][6] = {          /* Monday, Wednesday, Friday time */
        "08:00", "09:05", "10:10", "11:15", "12:20",
	"13:25", "14:30", "15:35", "17:15", "18:45", "20:15" };
    char trsx_day[9][6] = {     /* Tuesday, Thursday, Saturday, Sunday time */
        "08:00", "09:30", "11:00", "12:30",
	"14:00", "15:35", "17:15", "18:45", "20:15" };

    if ( strlen (argv[2]) == 1 && strlen (argv[3]) == 5 ) { /* day and time */
        switch (argv[2][0]) {     /* day argument */
	case 'M':
	    i = 0;
	    break;
	case 'W':
	    i = 1;
	    break;
	case 'F':
	    i = 2;
	    break;
	case 'T':
	    i = 3;
	    break;
	case 'R':
	    i = 4;
	    break;
	case 'S':
	    i = 5;
	    break;
	case 'X':
	    i = 6;
	    break;
	default:
	    fprintf (stderr, "day should be one of M, T, W, R, F, S, X\n");
	    remove_lock();               /* remove lock file before exit */
	    exit(ERROR);
	}

        if ( i < 3 ) {        /* When M or W or F */
	    for (j=0; j<11; j++) {
	        if ( !strcmp (mwf_day[j], argv[3]) ) {
		    index = i * 11 + j;               /* calculate index */
		    return (index);
		}
	    }
	    fprintf (stderr, "%s cannot be a time slot of MWF\n", argv[3]);
	    return (ERROR);
	}
	else {                /* when T or R or S or X */
	    for (j=0; j<9; j++) {
	        if ( !strcmp (trsx_day[j], argv[3]) ) {
		    index = 33 + ( i - 3 ) * 9 + j;    /* 33 MWF + TRSX */
		    return (index);
		}
	    }
	    fprintf (stderr, "%s cannot be a time slot of TRSX\n", argv[3]);
	    return (ERROR);
	}
    }

    print_usage();
    return (ERROR);

}

/*======================================================================
  Function course_check
  Input : char string of course name
  Output : error of normal return
  check validity of course name
======================================================================*/
int course_check (char *course) {

    int len;

    len = strlen (course);
    if ( len != 7 ) {
        fprintf(stderr, "Invaild course_number\n");
	remove_lock();
	exit(ERROR);
    }
    
    /* is course name llldddd ? (l : letter d : digit) */
    if ( isalpha((int)course[0]) && isalpha((int)course[1]) &&
	 isalpha((int)course[2]) && isdigit((int)course[3]) &&
	 isdigit((int)course[4]) && isdigit((int)course[5]) &&
	 isdigit((int)course[6] )) {
        return (NORMAL);
    }
    else {
        fprintf(stderr, "course_number must be formatted as llldddd\n");
	remove_lock();
	exit (ERROR);
    }
    
    return (NORMAL);

}




