/***********************************************************************
  Main function of Assignment # of Advanced Unix Programming.
  Input : options, days, times, course number, user name
  Output : list of current reserved desk database.
  Assumption : Course number does not check.
               Total reserved blocks by a user does not check.
	       Environment setting does not effort.
  Author : Jungkee (Jake) Kim
  Date : September 18, 2000
************************************************************************/

#include <errno.h>
#include <time.h>

#include "resoff.h"
#include "checks.h"

int main(int argc, char **argv) {

    int option;                       /* kind of options */
    int lfd;                          /* lock file descriptor */
    extern uid_t uid;                 /* user id */
    extern uid_t euid;                /* effective user id */
    char filename[256];               /* database file name */
    extern char lockfile[256];        /* lock file name */
    struct passwd *home;              /* passwd struct for home directory */
    time_t start_time;                /* start time of program */
    time_t current_time;              /* current time */

    option = get_option(argc, argv);  /* analyze input options */
    if ( !option ) {
        print_usage();
	exit(-1);
    }

    uid = getuid();
    euid = geteuid();

    home = getpwuid(euid);            /* module to get database file name */
    strcpy(filename, home->pw_dir);
    strcat(filename, "/resoff.db");

    strcpy(lockfile, home->pw_dir);   /* module to get lock file name */
    strcat(lockfile, "/resoff.lock");

    lfd = open (lockfile, O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);

    time (&start_time);
    current_time = start_time;

    /* wating until making lock file or maximum wait time */
    while ( lfd < 0 && current_time - start_time < MAX_WAIT ) {
        if ( errno == EEXIST ) {      /* only wait when file exist */
	    sleep(1);                 /* one second */
	}
	else {
	    fprintf(stderr, "lock file open error except noexist\n");
	    exit (ERROR);
	}
	lfd = open (lockfile, O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
	time (&current_time);
    }
    
    if ( lfd < 0 ) {
        fprintf(stderr, "unsuccessful getting resource within limited time\n");
	exit(ERROR);
    }
    
    /* Commented out codes for limit lock waiting test
    printf("lock\n");
    sleep (35);
    */

    switch( option ) {
    case _RESERVE:
        reserve(filename, argv);
        break;
    case _CANCEL:
        cancel(filename, argc, argv);
        break;
    case _LIST:
        list(filename);
	break;
    default:
        break;
    }

    remove_lock();              /* removing lock file */

    return NORMAL;

}





