#include "resoff.h"
#include "checks.h"
#include "mask.h"

int cancel (char *filename, int argc, char **argv) {

    extern int fd;
    int desk_number;
    int db_index;
    int loc;
    int len;
    Room slot;
    sigset_t newset, oldset;

    db_index = get_db_index (argv);
    if ( db_index < 0 ) {
        remove_lock();
        exit (ERROR);
    }
    loc = db_index * sizeof(slot);

    CHECK (fd = open (filename, O_CREAT | O_RDWR,
		      S_IRUSR | S_IWUSR));

    if ( empty_db (fd) ) {
        printf("Empty database, please reserve first\n");
	CHECK (unlink (filename));
	remove_lock();
	exit (ERROR);
    }

    SIGFILL(newset);
    BLOCK(newset, oldset);
    CHECK (lseek (fd, loc, SEEK_SET));
    CHECK (len = read (fd, &slot, sizeof(slot)));
    UNBLOCK(newset, oldset);

    desk_number = get_desk_number(slot, argc, argv);

    if ( desk_number < 4 ) {
        slot.count--;
	slot.empty[desk_number] = TRUE;
	BLOCK(newset, oldset);
	CHECK (lseek (fd, loc, SEEK_SET));
	CHECK (len = write (fd, &slot, sizeof(slot)));
	BLOCK(newset, oldset);
    }

    CHECK (close (fd));
    
    return (NORMAL);

}

int get_desk_number(Room slot, int argc, char **argv) {

    int i;
    extern uid_t uid;
    extern uid_t euid;
    uid_t target_uid;
    struct passwd *puid;

    if ( argc == 4 ) {
        target_uid = uid;
        for (i=0; i<4; i++) {
	    if ( !slot.empty[i] ) {
	        if ( target_uid == slot.desk[i].uid ) {
		    return (i);
		}
	    }
	}
	printf("There is no reservation to cancel for you\n");
    }
    else if ( argc ==5 ) {
	puid = getpwnam(argv[4]);
	if ( puid == NULL ) {
	    printf("No \"%s\" username exist\n", argv[4]);
	    remove_lock();
	    exit(ERROR);
	}
	target_uid = puid->pw_uid;
	if ( uid == euid || target_uid == uid ) {
	    for (i=0; i<4; i++) {
	        if ( !slot.empty[i] ) {
	            if ( target_uid == slot.desk[i].uid ) {
		        return (i);
		    }
		}
	    }
	    fprintf(stderr, "No %s\'s reservation at %s on %s\n",
		    argv[4], argv[3], argv[2]);
	}
	else {
	    fprintf(stderr, 
		   "You are not allowed to cancel other user's reservation\n");
	}
    }

    return (9999);

}
