#include "resoff.h"
#include "checks.h"
#include "mask.h"

/*======================================================================
  Function validity
  Input : database index, a time block data, course name
  Output : error message or return normally
  check validity of given options - cannot reserve more than 3 a day
                                    cannot reserve the same course
======================================================================*/
int validity (int db_index, Room slot, char *course) {

    int i;
    extern uid_t uid;                /* user id */

    if ( slot.count != 0 ) {         /* if block is not empty */
        for (i=0; i<4; i++) {
	    if ( !slot.empty[i] ) {  /* if empty desk */
	        if ( slot.desk[i].uid == uid ) {   /* the same user id */
		    fprintf(stderr, "you don't have to reserve twice\n");
		    remove_lock();
		    exit(ERROR);
		}                              /* the same course name */
		else if ( !strcmp(slot.desk[i].class, course) ) {
		    fprintf(stderr, "conflict course reservation\n");
		    remove_lock();
		    exit(ERROR);
		}
	    }
	}
    }

    check_day_reserve (db_index);   /* more than three reservation a day? */
    
    return (NORMAL);
  
}

/*======================================================================
  Function check_day_reserve
  Input : database index
  Output : error message or return normally
  check validity of given options - cannot reserve more than 3 a day
======================================================================*/
void check_day_reserve (int db_index) {

    int rcount = 0;                 /* resevation number of a user */
    int i, j;                       /* loop index */
    int begin, max_slot;            /* begin index, maximum block of a day */
    int loc;                        /* location of a block */
    int len;                        /* number of bytes read */
    int day;                        /* day index */
    sigset_t newset, oldset;        /* new and old signal sets */

    Room slot;                      /* a time block */
    extern uid_t uid;               /* user id */
    extern int fd;                  /* file description of database file */
    
    day = get_day (db_index);       /* calculate a day from index */
    if ( day < 0 ) {                /* unreachable value */
        fprintf(stderr, "db_index error\n");
	remove_lock();
	exit(ERROR);
    }

    switch (day) {
    case 0:                         /* Monday */
        begin = 0;
	max_slot = 11;
	break;
    case 1:                         /* Wednesday */
        begin = 11;
	max_slot = 11;
	break;
    case 2:                         /* Friday */
        begin = 2 * 11;
	max_slot = 11;
	break;
    case 3:                         /* Tuesday */
        begin = 33;
	max_slot = 9;
	break;
    case 4:                         /* Thursday */
        begin = 33 + 9;
	max_slot = 9;
	break;
    case 5:                         /* Saturday */
        begin = 33 + 9 * 2;
	max_slot = 9;
	break;
    case 6:                         /* Sunday */
        begin = 33 + 9 * 3;
	max_slot = 9;
	break;
    default:
        fprintf (stderr, "illigal db_index error\n");
	remove_lock();
	exit(ERROR);
	break;
    }

    SIGFILL(newset);

    /* counting the number of reservaion of a user on a day */
    for ( i = begin; i < begin + max_slot; i++ ) {
        loc = i * sizeof (slot);                    /* block location */
	BLOCK(newset, oldset);
	CHECK (lseek (fd, loc, SEEK_SET));
	CHECK (len = read (fd, &slot, sizeof(slot)));
	UNBLOCK(newset, oldset);
	if ( slot.count > 0 ) {
	    for (j=0; j<4; j++) {
	        if ( !slot.empty[j] ) {         /* if desk is not empty */
		    if ( slot.desk[j].uid == uid ) {
		        rcount++;
		    }
		}
	    }
	}

	if ( rcount > 2 ) {
	  fprintf(stderr, "you cannot reserver more than three slots a day\n");
	  remove_lock();                        /* remove the lock file */
	  exit(ERROR);
	}
    }

}
