/* file: callp2.c
   call specified program with specified userid
 */   

#define _XOPEN_SOURCE 1
#define _XOPEN_SOURCE_EXTENDED 1

#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/resource.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <wait.h>

#define MAXARGS 10
#define PROGNAME "resoff"
#define TIMEOUT 10

char usage[] = 
   "usage: callp2 userid p2_args...\n";

/* this null handler procedure allows the signal to be handled
   and return with no other direct effect */
void handler() {
  ;
}

int 
main( int argc, char ** argv) {
   int result;
   int i;
   uid_t userid;
   char *tmp1, *tmp2;
   char *myenv[MAXARGS+2];
   char *myargv[MAXARGS+2];
   /* struct rlimit rlim; */
   mode_t oldmask;
   pid_t child;
   int status;
   struct sigaction act;
   char dirbuf[1024];
   char *pathbuf;

   if (argc > MAXARGS) {
     fprintf (stderr, "too many arguments\n");
     exit (-1);
   }
   if (argc < 3) {
     fprintf (stderr, usage);
     exit (-1);
   }

   userid = atoi (argv[1]);
   if (userid == 0) {
      userid = getuid();
   }
   if (userid == 0) {
      fprintf (stderr, "invalid userid=0\n");
      exit (-1);
   }

   if (getcwd (dirbuf, 1024) == NULL) {
      perror ("callp2: getcwd");
      exit (-1);
   }

   if ((tmp1 = getenv ("RESOFFDIR"))) {
      tmp2 = malloc (strlen (tmp1) + strlen ("RESOFFDIR=") + 1);
      strcpy (tmp2, "RESOFFDIR=");
      strcat (tmp2, tmp1);
      myenv[0] = tmp2;
      myenv[1] = NULL;
   } else myenv[0] = NULL;

   for (i = 0; i < (argc-1); i++) myargv[i] = argv[i+1];
   pathbuf = (char *) malloc (strlen (dirbuf) + strlen (PROGNAME) + 2);
   strcpy (pathbuf, dirbuf);
   strcat (pathbuf, "/");
   strcat (pathbuf, PROGNAME);
   myargv[0] = PROGNAME;
   myargv[argc-1] = NULL;

   oldmask = umask (0);
   if (chmod (pathbuf, S_ISUID | S_IRUSR | S_IWUSR |
              S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)) {
      perror ("callp2: chmod");
      fprintf (stderr, "path=%s\n", pathbuf);
      exit (-1);
   }

   if (setuid (userid)) {
      perror ("callp2: setuid");
      fprintf (stderr, "userid=%d\n", (int) userid);
      exit (-1);
   }
   if (! getuid()) {
      fprintf (stderr, "callp2: userid=%d\n", (int) userid);
      exit (-1);
   }

   if (sigaction (SIGALRM, NULL, &act)) {
      perror ("callp2: sigaction (1)");
      exit (-1);
   }
   act.sa_handler = handler;
   if (sigaction (SIGALRM, &act, NULL)) {
      perror ("callp2: sigaction (2)");
      exit (-1);
   } 

   /* Original idea was to limit child resource usage
      to reduce danger of runaway processes.
      Difficulty was trying to find appropriate limits,
      especially limits that make sense across different
      operating sytems.
   getrlimit (RLIMIT_CPU, &rlim);
   rlim.rlim_cur = 10;
   if (setrlimit (RLIMIT_CPU, &rlim)) {
     perror ("callp2: setrlimit (RLIMIT_CPU)");
   }
#ifdef RLIMIT_NPROC
   getrlimit (RLIMIT_NPROC, &rlim);
   rlim.rlim_cur = 1;
   if (setrlimit (RLIMIT_NPROC, &rlim)) {
     perror ("callp2: setrlimit (RLIMIT_NPROC)");
   }
#endif
   getrlimit (RLIMIT_NOFILE, &rlim);
   rlim.rlim_cur = 10;
   if (setrlimit (RLIMIT_NOFILE, &rlim)) {
     perror ("callp2: setrlimit (RLIMIT_NOFILE)");
   }
   getrlimit (RLIMIT_AS, &rlim);
   rlim.rlim_cur = 10000;
   if (setrlimit (RLIMIT_AS, &rlim)) {
     perror ("callp2: setrlimit (RLIMIT_AS)");
   }
   getrlimit (RLIMIT_FSIZE, &rlim);
   rlim.rlim_cur = 10000;
   if (setrlimit (RLIMIT_FSIZE, &rlim)) {
     perror ("callp2: setrlimit (RLIMIT_FSIZE)");
   }
   getrlimit (RLIMIT_STACK, &rlim);
   rlim.rlim_cur = 1000;
   if (setrlimit (RLIMIT_STACK, &rlim)) {
     perror ("callp2: setrlimit (RLIMIT_STACK)");
   }
   getrlimit (RLIMIT_DATA, &rlim);
   rlim.rlim_cur = 1000;
   if (setrlimit (RLIMIT_DATA, &rlim)) {
     perror ("callp2: setrlimit (RLIMIT_DATA)");
   }
   end of suppressed code */
   printf ("[");
   for (i=0; myargv[i]; i++) printf ("%s ", myargv[i]);
   printf ("] with  environment: ");
   for (i=0; myenv[i]; i++) printf ("%s ", myenv[i]);
   printf ("\n");
   fflush (stdout);
   fflush (stderr);
   if ((child = fork())) {
      /* this is the parent process */
      alarm (TIMEOUT);
      result = waitpid (child, &status, 0);
      if (result == -1) {
        if (errno == EINTR) {
           if (kill (child, SIGKILL)) {
              perror ("callp2: kill");
              exit (-1);
           }
           fprintf (stderr, "hung process killed after %d seconds\n",
            TIMEOUT);
           exit (-1);
        } else {
           perror ("callp2: waitpid");
           exit (-1);
        }
      } else {
        exit (WEXITSTATUS (status));
      };
   };
   /* this is the child process */
   fprintf (stderr, "pathbuf=%s\n", pathbuf);
   result = execve (pathbuf, myargv, myenv);
   perror ("callp2: execve");
   exit (-1);      
}








