/* newproc.c
   ---------
   Illustration of child-side view of processes
   and receiver-side operations on signals.
   This program is intended for use as child process of program proc.c.
 */

#define _POSIX_C_SOURCE 199506L
#include "mine.h"

extern char **environ;

void usage () {
   fprintf (stderr, "usage: newproc <letter_option>\n");
   fprintf (stderr, "<letter_option> meanings:\n");
   fprintf (stderr, "  a = just exit normally\n");
   fprintf (stderr, "  b = exit with status 0\n");
   fprintf (stderr, "  c = exit with status 1\n");
   fprintf (stderr, "  d = suspend for SIGUSR1 or SIGUSR2, then exit\n");
   fprintf (stderr, "  e = %ds alarm with handler, pause\n", HOWLONG);
   fprintf (stderr, "  f = %ds alarm with no handler, pause\n", HOWLONG);
   fprintf (stderr, "  g = just print out environment variable values\n");
   fprintf (stderr, "SIGUSR1 is always masked\n");
   fflush (stderr); /* flushing stderr should be redundant */
   exit (-1);
}

void handler (int sig, siginfo_t * info, void * context) {
   fprintf (stderr, "handler executing for signal %d\n", sig);
   fflush (stderr); /* flushing stderr should be redundant */
}

int main(int argc, char **argv) {

   sigset_t emptyset, setalrm, set1, set2, old_set;
   struct sigaction new_act, old_act;

   /* mask the signal SIGUSR1 */
   CHECK (sigemptyset (&set1));
   CHECK (sigaddset (&set1, SIGUSR1));
   CHECK (sigprocmask (SIG_SETMASK, &set1, &old_set));

   /* prepare a set to wait for for SIGUSR2 and SIGUSR1 */
   CHECK (sigemptyset (&set2));
   CHECK (sigaddset (&set2, SIGUSR1));
   CHECK (sigaddset (&set2, SIGUSR2));

   /* set up variables used later to
      install a handler for SIGALRM */
   CHECK (sigemptyset (&setalrm));
   CHECK (sigaddset (&setalrm, SIGALRM));
   CHECK (sigemptyset (&emptyset));
   new_act.sa_flags = SA_SIGINFO;
   new_act.sa_sigaction = handler;
   new_act.sa_mask = emptyset;

   if (argc != 2) usage();
   fprintf (stderr, "newproc %s: our ID = %d; parent ID = %d\n",
      argv[1], (int) getpid(), (int) getppid());
   fflush (stderr); /* flushing stderr should be redundant */

   switch (argv[1][0]) {
   case 'a' :
      break;
   case 'b' :
      exit (0);
   case 'c' :
      exit (1);
   case 'd' :
      /* install handler for SIGUSR2 */
      CHECK (sigaction (SIGUSR2, &new_act, &old_act));
      /* arrange to ignore SIGUSR1 */
      new_act.sa_handler = SIG_IGN;
      CHECK (sigaction (SIGUSR1, &new_act, &old_act));
      /* wait until handler executes */
      fprintf (stderr, "process %d waiting for SIGUSR2, ignoring SIGUSR1\n",
        (int) getpid());
      sigsuspend (&emptyset);
      fprintf (stderr, "process %d awaking from sigsuspend\n",
        (int) getpid());
      break;
   case 'e' :
      /* install handler for SIGALARM */
      CHECK (sigaction (SIGALRM, &new_act, &old_act));
      /* generate alarm signal in HOWLONG seconds */
      alarm (HOWLONG);
      pause ();
      break;
   case 'f' :
      /* generate alarm signal in HOWLONG seconds */
      alarm (HOWLONG);
      pause ();
      break;
   case 'g' : {
      char **p; int i;
      fprintf (stderr, "environment values:\n");
      for (p = environ, i = 0; *p; p++, i++) {
         fprintf (stderr, "%3d: %s\n", i, *p);
      }}
      break;
   default :
      usage();
   }
   return 0;   
}





