/* proc.c
   ------
   Illustration of parent-side view of processes.
 */

#include "mine.h"

void usage () {
   fprintf (stderr, "usage: proc <letter_option>\n");
   fprintf (stderr, "<letter_option> is normally passed to child:\n");
   fprintf (stderr, "  a = just exit normally\n");
   fprintf (stderr, "  b = exit with status 0\n");
   fprintf (stderr, "  c = exit with status 1\n");
   fprintf (stderr, "  d = suspend for SIGUSR1 or SIGUSR2, then exit\n");
   fprintf (stderr, "  e = %ds alarm with handler, pause\n", HOWLONG);
   fprintf (stderr, "  f = %ds alarm with no handler, pause\n", HOWLONG);
   fprintf (stderr, "  g = just print out environment variable values\n");
   fprintf (stderr, "  x = all of the above\n");
   fflush (stderr);
   exit (-1);
}

void check_status (pid_t child_id, int status) {
   fprintf (stderr, "process %d ", (int) child_id);
   if (WIFEXITED (status)) {
      fprintf (stderr,
        "exited with status %d\n", WEXITSTATUS (status));
   }else if (WIFSIGNALED (status)) {
      fprintf (stderr,
        "terminated by signal %d\n", WTERMSIG (status));
   }else if (WIFSTOPPED (status)) {
      fprintf (stderr,
        "stopped by signal %d\n", WSTOPSIG (status));
   }else {
      fprintf (stderr,
        "unclassified status %d\n", status);
   }
   fflush (stderr);
}

int main(int argc, char **argv) {
   
   char child_pathname[] = "./newproc";
   char *child_argv[3];
   char *child_env[4];
   pid_t child_id;
   int child_status;
   
   if (argc != 2) usage();

   fprintf (stderr, "proc %s: our ID = %d\n", argv[1], (int) getpid());
   fflush (stderr);

   child_argv[0] = child_pathname;
   child_argv[1] = argv[1];
   child_argv[2] = NULL;

   child_env[0] = (char *) malloc (sizeof (argv[1] + 8));
   if (! child_env[0]) ERR_QUIT ("out of memory");
   child_env[0][0] = '\0';
   strncat (child_env[0], "argv[1]=", 8);
   strncat (child_env[0], argv[1], sizeof (argv[1]));
   child_env[1] = "I_AM_CHILD";
   child_env[2] = NULL;

   if (argv[1][0] != 'x') {
      if ( (child_id = fork())) { /* I am the parent */
         CHECK (waitpid (child_id, &child_status, 0));
         check_status (child_id, child_status);
      } else { /* I am the child */
         CHECK (execve (child_pathname, child_argv,
           child_env));
      }
   } else { /* execute all child processes */
      char c;
      for (c = 'a'; c <= 'g'; c++) {
         if ( (child_id = fork())) { /* parent */
            if (c == 'd') {
               sleep (2);
               kill (child_id, SIGKILL);
            }
         } else { /* child */
            child_argv[1][0] = c;
            child_env[0][9] = c;
            CHECK (execve (child_pathname, child_argv,
              child_env));
         }
      }
      for (;;) {
         child_id = waitpid (-1, &child_status, 0);
         if (child_id == -1) {
            if (errno == EINTR) {}
            else if (errno == ECHILD) break;
            else perror ("wait");
         } else {
            check_status (child_id, child_status);
         }
      }
   }

   return 0;
}





