/* barbers.c

   @(#)barbers.c	1.1 9/6/94

   Main program, for Barbers Problem exercise

   Prepared for COP4610, Fall 1994
   --Jeff Bauer

*/

#include <stdio.h>
#include <ctype.h>
#include <pthread.h>
#include "barbers.h"	/* contains definitions of *CHAIRS* and *MAXCUST* */

/* Insert your global constants and variables here.
 */

int numcust;		/* number of customers that will enter barbershop */
int waiting;		/* # of customers currently waiting for haircut */
int livecust;         	/* # of live customers */

pthread_cond_t	barber_Q;
pthread_mutex_t lock;		/* for mutex on shared data */
pthread_mutex_t outlock;	/* for mutex on printf() data */

/* Here's predefined empty C functions to help you lay out the program.
 */

void BarbershopInit(void)	/* Initialization routine */
{

/* Insert any initialization code here */

   waiting = 0;
   pthread_mutex_init(&lock, NULL);
   pthread_mutex_init(&outlock, NULL);
   pthread_cond_init(&barber_Q);
}

void trace(char *s)

  /* Print out message, for use in execution tracing.  Serialize the execution.
     s = message 
   */
{
  pthread_mutex_lock(&outlock);
  fprintf(stdout,"%s\n",s);
  pthread_mutex_unlock(&outlock);
}

void Barber(void)		/* Barber routine (an independent thread) */
{
  char tbuf[128];

/* Insert the barber code here.  There will be one instance of this
   thread. */

   pthread_mutex_lock(&lock);
   while (livecust > 0) {
     trace("Barber : Sleeping");
     while (waiting == 0)
       pthread_cond_wait(&barber_Q, &lock);	/* sleep until needed */
     waiting = waiting - 1;		/* one less customer is now waiting */
     sprintf(tbuf, "Barber : cutting customer's hair (%d still waiting)", 
       waiting);
     trace(tbuf);
   }
   pthread_mutex_unlock(&lock);
}

void Customer(int *custno)	/* Customer routine (one or more independent
				   threads) */
{
  int self;
  char tbuf[128];

  self = *custno;
  sprintf(tbuf, "Customer %d : Entering barbershop.", self);
  trace(tbuf);

......This might be more interesting if customer had an internal loop,
......maybe with a time-delay (say, using pthread_cond_timedwait).
......In general, the shut-down synchronization turns out to be one of
......more interesting aspects of such problems.  For example, the
......barber might decide to close down his shop.  At that point, 
......new customers would need to start turning away new customers,
......clear out the waiting ones, and then shut himself down.
......That could be a variation, if we use this problem again some
......other term, to cut down on carbon-copy solutions.

/* Insert the customer code here.  There will be *numcust* instances of
   this thread active (up to a maximum of MAXCUST).  Be sure to create
   this thread and pass it it's instance number, so you can distinguish
   one customer from another. */

   pthread_mutex_lock(&lock);		/* serialize access to *waiting* */
   if (waiting < CHAIRS) {	
......The test is right with "<", but need to move the increment of waiting
......up, before the trace output
     sprintf(tbuf, "Customer %d : waiting for a barber (%d total waiting).", 
        self, waiting);
     trace(tbuf);
     waiting = waiting + 1;		/* one more customer is waiting */
     pthread_cond_signal(&barber_Q);	/* force barber to re-evaluate */
     sprintf(tbuf, "Customer %d : getting a haircut and leaving.", self);
     trace(tbuf);
   }
   else {
     sprintf(tbuf, "Customer %d : shop is full, leaving without a haircut.", 
	self);
     trace(tbuf);
   }
   livecust = livecust - 1;
   pthread_mutex_unlock(&lock);		/* release access to *waiting* */
   pthread_exit(0);
}

main()
{
  /* Insert your local declarations here.
   */

   int i;
   int cust_no[MAXCUST];
   pthread_t barber_th;
   pthread_t cust_th[MAXCUST];

  /* Insert your main program code here.
   */

  /* Accept and do rudimentary validation on the number of customers.
   */

  do {
    printf("Enter the number of customers->");
    scanf("%d", &numcust);
  }
  while ((numcust < 1) || (numcust > MAXCUST));
  livecust = numcust;

  /* Initialization any global data structures, if required.
   */

  BarbershopInit();

  /* To do here : basic threads management - create the threads and wait
     for them to finish before exiting.
   */

  /* Start up the barber thread */

   pthread_create(&barber_th, NULL, (pthread_func_t) Barber, NULL);

  /* Start up the customer threads */

  for (i = 0; i < numcust; i++) {
    cust_no[i] = i;
    pthread_create(&cust_th[i], NULL, (pthread_func_t) Customer, 
      (int *) &cust_no[i]);
  }

  /* Wait for all the threads to shut down */

  for (i = 0; i < numcust; i++) pthread_join(cust_th[i], NULL);

  pthread_join(barber_th, NULL);

.....Drop the following "detach" calls, since threads are already "joined".
.....Generally, you either detach or join a thread, but not both.
.....This is an error I made in one of my early examples, which you may have
.....seen.  I only noticed it when I (later) checked the return value
.....from the pthread_detach() call and found it to be non-zero.

  printf("barber shop is closed.\n");

  /* Allow collection of the resources of the terminated threads */

  for (i = 0; i < numcust; i++) {
    pthread_detach(&cust_th[i]);
  }

  pthread_detach(&barber_th[i]);

  return 0;
}

--
Sample output :

Enter the number of customers->16
Barber : Sleeping
Customer 0 : Entering barbershop.
Customer 1 : Entering barbershop.
Customer 2 : Entering barbershop.
Customer 3 : Entering barbershop.
Customer 0 : waiting for a barber (0 total waiting).
Customer 4 : Entering barbershop.
Customer 5 : Entering barbershop.
Customer 6 : Entering barbershop.
Customer 7 : Entering barbershop.
Customer 0 : getting a haircut.
Customer 8 : Entering barbershop.
Customer 9 : Entering barbershop.
Customer 10 : Entering barbershop.
Customer 11 : Entering barbershop.
Customer 0 : leaving the shop after getting a haircut.
Customer 12 : Entering barbershop.
Customer 13 : Entering barbershop.
Customer 14 : Entering barbershop.
Customer 15 : Entering barbershop.
Customer 1 : waiting for a barber (1 total waiting).
Customer 1 : getting a haircut.
Customer 1 : leaving the shop after getting a haircut.
Customer 2 : waiting for a barber (2 total waiting).
Customer 2 : getting a haircut.
Customer 2 : leaving the shop after getting a haircut.
Customer 3 : waiting for a barber (3 total waiting).
Customer 3 : getting a haircut.
Customer 3 : leaving the shop after getting a haircut.
Barber : cutting customer's hair (3 still waiting)
Customer 4 : waiting for a barber (3 total waiting).
Customer 4 : getting a haircut.
Customer 4 : leaving the shop after getting a haircut.
Customer 5 : waiting for a barber (4 total waiting).
Customer 5 : getting a haircut.
Customer 5 : leaving the shop after getting a haircut.
Customer 6 : shop is full, leaving w/out a haircut.
Customer 7 : shop is full, leaving w/out a haircut.
Customer 8 : shop is full, leaving w/out a haircut.
Customer 9 : shop is full, leaving w/out a haircut.
Customer 10 : shop is full, leaving w/out a haircut.
Customer 11 : shop is full, leaving w/out a haircut.
Customer 12 : shop is full, leaving w/out a haircut.
Customer 13 : shop is full, leaving w/out a haircut.
Customer 14 : shop is full, leaving w/out a haircut.
Customer 15 : shop is full, leaving w/out a haircut.
Barber : Sleeping
Barber : cutting customer's hair (4 still waiting)
Barber : Sleeping
Barber : cutting customer's hair (3 still waiting)
Barber : Sleeping
Barber : cutting customer's hair (2 still waiting)
Barber : Sleeping
Barber : cutting customer's hair (1 still waiting)
Barber : Sleeping
Barber : cutting customer's hair (0 still waiting)
Barber : Sleeping

