/* example of single producer and multiple consumers
 */

#define _POSIX_C_SOURCE 199506L
#define _REENTRANT
#include <string.h>
#include <pthread.h>
#include <stdio.h>
#include <values.h>
#include <errno.h>

#define BUF_SIZE 10
int buf[BUF_SIZE];
int next_in = 0, next_out = 0;

#define N_CONSUMERS 5
int arguments[N_CONSUMERS];
/* arguments[i] = i */
pthread_t consumer_id[N_CONSUMERS];
pthread_t producer;

void * consumer_body (void *arg) {
/* create one unit of data and put it in the buffer
   Assumes arg points to an element of the array id_number,
   identifying the current thread.
 */
  int tmp;
  int self = *((int *) arg);

  fprintf(stderr, "consumer thread starts\n"); 
  for (;;) {
     /* take one unit of data from the buffer
      */
     while (next_out == next_in) thr_yield(); /* wait for buffer to fill */
     tmp = buf[next_out];
     next_out = (next_out + 1) % BUF_SIZE;     
     /* copy the data to stdout
      */
     fprintf (stdout, "thread %d: %d\n", self, tmp);
  }
  fprintf(stderr, "consumer thread exits\n"); 
}
void * producer_body (void * arg) {
/* takes one unit of data from the buffer
 */
   int i;
   fprintf(stderr, "producer thread starts\n"); 
   for (i = 0; i < MAXINT; i++) {
     /* put value i into the buffer
      */
     while (((next_in + 1) % BUF_SIZE) == next_out) thr_yield(); /* wait for space in buffer */
     buf[next_in] = i;
     next_in = (next_in + 1) % BUF_SIZE;     
  }
}

int main () {
   int i, result;
   pthread_attr_t attrs;

   /* use default attributes */
   pthread_attr_init (&attrs);

   /* create producer thread */
   if (result = pthread_create (
          &producer, /* place to store the id of new thread */
          &attrs,
          producer_body,
          NULL))  /* no need for argument */ {
      fprintf (stderr, "pthread_create: %d\n", result);
      exit (-1);
   } 
   fprintf(stderr, "producer thread created\n"); 
   /* create consumer threads */
   for (i = 0; i < N_CONSUMERS; i++) {
      arguments[i] = i;
      if (result = pthread_create (
          &consumer_id[i],
          &attrs,
          consumer_body,
          &arguments[i])) {
        fprintf (stderr, "pthread_create: %d\n", result);
        exit (-1);
      } 
   }
   fprintf(stderr, "consumer threads created\n");
   /* give the other threads a chance to run */
   sleep (10);
}



