/* smokers.c
   %W% %G%

   Main program, for Smokers Problem exercise

   The effect of this program is to read an integer from the
   standard input, indicating an upper bound on how many trace-steps
   to allow before starting to shut down.  It will allow this many
   steps, and then shut down.  The shutdown will take a few more
   steps.  While the program is running, it will output a series
   of lines.  Each line begins with a number identifying an action by
   one of the smokers or the dealer.

   Prepared for COP4610, Spring 1994
   --Ted Baker

 */

#include <stdio.h>
#include <ctype.h>
#include <pthread.h>
#include "choose.h"

pthread_mutex_t lock;         /* protects shared state */
pthread_cond_t  cond[N];      /* th[i] waits on cond[i] */
int step_count, maxsteps;     /* number of steps to run this test */
int smoking_count;            /* should be zero or one */

void wait(int self)
{ pthread_mutex_lock(&lock);
  pthread_cond_signal(&cond[VENDOR]);
  if (step_count<maxsteps) {
    pthread_cond_wait(&cond[self], &lock);
  }else{
    exited[self] = 1;
    smoker_exit_msg(self);
    pthread_mutex_unlock(&lock);
    pthread_exit(0);
  }
  pthread_mutex_unlock(&lock);
}

void serve(int choice)
{ pthread_mutex_lock(&lock);
  pthread_cond_signal(&cond[choice]);
  pthread_cond_wait(&cond[VENDOR],&lock);
  pthread_mutex_unlock(&lock);
}

void * smoker(void *arg)
{ int self= *(int *)arg;
  for (;;) {
    smoker_wait_msg(self);
    wait(self);
    smoking_count++;
    if (smoking_count!=1) printf("*** more than one smoker\n");
    smoking_count--;
    smoker_done_msg(self);
    step_count++;    
  }
}

void * vendor(void *arg)
{ int self = *(int *)arg;
  int choice;
  while ((choice = choose())!=0) {
    vendor_service_msg(choice);
    serve(choice);
  }
  vendor_exit_msg();
  pthread_exit(0);
}

main()
{
  int i; 
  pthread_t th[N]; /* IDs of the philospher threads */
  int       no[N]; /* corresponding table position numbers*/

  /* initialize mutexes and condition variables */

  pthread_mutex_init(&lock, NULL);
  for (i = 0; i < N; i++) pthread_cond_init(&cond[i], NULL);

  fprintf(stdout,"enter number of steps to start shutdown:"); fflush(stdout);
  fscanf(stdin,"%d",&maxsteps);

  /* start up the vendor thread */
  no[0] = 0;
  pthread_create(&th[0], NULL, vendor, (int *) &no[0]);

  /* start up the smoker threads */
  for (i = 1; i < N; i++) {
    no[i] = i;
    pthread_create(&th[i], NULL, smoker, (int *) &no[i]);
  }

  /* wait for all the threads to shut down */
  for (i = 0; i < N; i++) pthread_join(th[i], NULL);

  return 0;
}







