/* example4.c -- T. P. Baker, 13 September 1999, 11 September 2000

   This is an illustration of the use of some Unix system calls.
 */

#include <stdio.h>
#include <strings.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
/* Why does the following have "..." instead of "<...>"? */
#include "checks.h"

int main (int argc, char **argv)
{

  /* Demonstration of operations to fetch
     real and effective user and group IDs.
   */

 { uid_t uid;   /* real user id */
   gid_t gid;   /* real group id */
   uid_t euid;  /* effective user id */
   gid_t egid;  /* effective group id */

   uid = getuid();
   printf ("uid = %d\n", (int) uid);

   gid = getgid();
   printf ("gid = %d\n", (int) gid);

   euid = geteuid();
   printf ("euid = %d\n", (int) euid);

   egid = getegid ();
   printf ("egid = %d\n", (int) egid);
 }

 /* Demonstration of direct random-access I/O,
    using lseek().
  */

 { int fd;
   char filename[] = "tmp_file";

   typedef struct {
     int A;
     char B[10];
     float C;
   } my_rec;

   my_rec buf;
   int  count;
   int  len;
   struct stat statbuf;
   ssize_t offset;
   int n;
   mode_t mask;

   /* set file creation mask to override default */
   CHECK (mask = umask (0));

   /* open or create the file */
   CHECK (fd = open (filename, O_CREAT | O_RDWR,
       S_IRUSR | S_IWUSR | S_IWGRP | S_IRGRP));

   /* find out the length of the file */
   CHECK (fstat (fd, &statbuf));
   if (statbuf.st_size) {
      /* The file already existed, so we can read
         the value of count from beginning of the file. */
      CHECK(len = read (fd, &count, sizeof (count)));
      printf ("count = %d\n", count);
      /* update count */
      count++;
      CHECK (lseek (fd, 0, SEEK_SET));
      len = write (fd, &count, sizeof (count));
      /* position to the last record in the file */
      CHECK (offset = lseek (fd, -sizeof(buf), SEEK_END));
      /* read the last record in the file */
      CHECK(len = read (fd, &buf, sizeof (buf)));
      printf ("A = %d, B = \"%s\", C = %f\n", buf.A, buf.B, buf.C);   
      /* update the buffer contents */
      buf.A++;
      /* convert the string buf.B to an integer n */
      sscanf (buf.B, "%d", &n);
      n++;
      /* convert the new value of n back into a string */
      sprintf (buf.B, "%d", n);
      buf.C++;
      /* add the new record to the end of the file */
      CHECK (len = write (fd, &buf, sizeof (buf)));
   } else {
      /* We just created the file, so we need to 
         initialize count at beginning of file and write
         the first record. */
      count = 1;
      CHECK (len = write (fd, &count, sizeof (count)));
      buf.A = 1;
      sprintf (buf.B, "%d", 1);
      buf.C = 1.0;
      CHECK (len = write (fd, &buf, sizeof (buf)));
   }
   CHECK (close (fd));

   return 0;
 }
}
