#define _POSIX_C_SOURCE 199506L

#include <ctype.h>
#include "resoff.h"
#include "checks.h"
#include "mask.h"

int reserve (char *filename, char **argv) {

    int i;
    int db_index;
    int loc;
    extern int fd;
    extern uid_t uid;
    int len;
    Room slot;
    sigset_t newset, oldset;

    course_check (argv[4]);

    db_index = get_db_index(argv);
    if ( db_index < 0 ) {
        remove_lock();
        exit(ERROR);
    }
    loc = db_index * sizeof(slot);
        
    CHECK (fd = open (filename, O_CREAT | O_RDWR,
		      S_IRUSR | S_IWUSR));

    if ( empty_db (fd) ) {
	initdb(filename);
    }

    SIGFILL(newset);
    BLOCK(newset, oldset);

    CHECK (lseek (fd, loc, SEEK_SET));
    CHECK (len = read (fd, &slot, sizeof(slot)));

    UNBLOCK(newset, oldset);

    if ( slot.count > -1 && slot.count < 4 ) {
        slot.count++;
	make_upper(argv[4]);
	validity(db_index, slot, argv[4]);
        for (i=0; i<4; i++) {
	    if ( slot.empty[i] ) {
	        slot.empty[i] = FALSE;
	        slot.desk[i].uid = uid;
		make_upper(argv[4]);
		strcpy(slot.desk[i].class, argv[4]);
		BLOCK(newset, oldset);
		CHECK (lseek (fd, loc, SEEK_SET));
		CHECK (len = write (fd, &slot, sizeof(slot)));
		CHECK (close(fd));
		UNBLOCK(newset, oldset);
		return (NORMAL);
	    }
	}
    }
    else if ( slot.count == 4 ) {
        fprintf(stderr, "No empty desk at %s on %s\n", argv[3], argv[2]);
	CHECK (close(fd));
	remove_lock();
	exit (ERROR);
    }
    else {	
	fprintf(stderr, "Slot count error in reserve.\n");
	CHECK (close(fd));
	remove_lock();
	exit (ERROR);
    }
    
    return(NORMAL);

}

int get_db_index (char **argv) {

    int i, j;
    int index;
    char mwf_day[11][6] = {
        "08:00", "09:05", "10:10", "11:15", "12:20",
	"13:25", "14:30", "15:35", "17:15", "18:45", "20:15" };
    char trsx_day[9][6] = {
        "08:00", "09:30", "11:00", "12:30",
	"14:00", "15:35", "17:15", "18:45", "20:15" };

    if ( strlen (argv[2]) == 1 && strlen (argv[3]) == 5 ) {
        switch (argv[2][0]) {
	case 'M':
	    i = 0;
	    break;
	case 'W':
	    i = 1;
	    break;
	case 'F':
	    i = 2;
	    break;
	case 'T':
	    i = 3;
	    break;
	case 'R':
	    i = 4;
	    break;
	case 'S':
	    i = 5;
	    break;
	case 'X':
	    i = 6;
	    break;
	default:
	    fprintf (stderr, "day should be one of M, T, W, R, F, S, X\n");
	    remove_lock();
	    exit(ERROR);
	}

        if ( i < 3 ) {        /* When M or W or F */
	    for (j=0; j<11; j++) {
	        if ( !strcmp (mwf_day[j], argv[3]) ) {
		    index = i * 11 + j;
		    return (index);
		}
	    }
	    fprintf (stderr, "%s cannot be a time slot of MWF\n", argv[3]);
	    return (ERROR);
	}
	else {
	    for (j=0; j<9; j++) {
	        if ( !strcmp (trsx_day[j], argv[3]) ) {
		    index = 33 + ( i - 3 ) * 9 + j;
		    return (index);
		}
	    }
	    fprintf (stderr, "%s cannot be a time slot of TRSX\n", argv[3]);
	    return (ERROR);
	}
    }

    return (ERROR);

}

int course_check (char *course) {

    int len;

    len = strlen (course);
    if ( len != 7 ) {
        fprintf(stderr, "Invaild course_number\n");
	remove_lock();
	exit(ERROR);
    }
    
    if ( isalpha((int)course[0]) && isalpha((int)course[1]) &&
	 isalpha((int)course[2]) && isdigit((int)course[3]) &&
	 isdigit((int)course[4]) && isdigit((int)course[5]) &&
	 isdigit((int)course[6] )) {
        return (NORMAL);
    }
    else {
        fprintf(stderr, "course_number must be formatted as llldddd\n");
	remove_lock();
	exit (ERROR);
    }
    
    return (NORMAL);

}




