/***********************************************************************
  Server main function of Assignment #3 of Advanced Unix Programming.
  Input : string queries from 'resoffc' clients through TCP/IP
  Output : list of current reserved desk database through TCP/IP.
  Assumption : Course number does not check.
               Total reserved blocks by a user does not check.
  Author : Jungkee (Jake) Kim
  Date : October 9, 2000 (Due Date)
************************************************************************/
#include "config.h"
#include "resoff.h"
#include "boolean.h"

int main()
{
    char *envpath;                    /* environment path name */
    char *envport;                    /* environment port number */
    char filename[256];               /* database file name */
    char lockfile[256];               /* lock file name */
    char recvline[30];                /* received command */
    struct passwd *home;              /* passwd struct for home directory */
    int lfd;                          /* lock file descriptor */
    int fd;                           /* file description of database file */
    int listenfd;
    int connfd;
    int n;                            /* number of bytes read */
    struct sockaddr_in cliaddr;       /* client socketaddr */
    struct sockaddr_in servaddr;      /* server socketaddr */
    socklen_t clilen;                 /* client value result */
    pid_t childpid;
    
    envpath = getenv ("RESOFFDIR");   /* get environment path */
    if ( envpath == NULL ) {  /* module to get database file name from home */
        home = getpwuid(geteuid());
	strcpy(filename, home->pw_dir);
	strcat(filename, "/resoff.db");
	strcpy(lockfile, home->pw_dir);   /* module to get lock file name */
	strcat(lockfile, "/resoff.lock");
    }
    else {                    /* module to get database file name from env */
        strcpy(filename, envpath);
	strcat(filename, "/resoff.db");
	strcpy(lockfile, envpath);        /* module to get lock file name */
	strcat(lockfile, "/resoff.lock");
    }
    
    lfd = open (lockfile, O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
    
    if ( lfd < 0 ) {
        ERR_QUIT("Other server is running or lock is not deleted "
                 "after termination.\n");
    }
    
    fd = open (filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);

    if ( fd < 0 ) {
        openerrmsg();
        remove_lock(lockfile);
        exit(ERROR);
    }
    
    initdb(fd);
    close(fd);
    
    if ( (listenfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
        socketerrmsg();
        exit(ERROR);
    }
    
    memset (&servaddr, 0, sizeof(servaddr));
    servaddr.sin_family = AF_INET;
    servaddr.sin_addr.s_addr = htonl(INADDR_ANY);

    envport = getenv ("RESOFFPORT");       /* get environment port number */
    if ( envport == NULL ) {
        servaddr.sin_port = htons(SERV_PORT);
    }
    else {              /* check the validity of port number: all number? */
        if ( is_all_digit(envport) ) {
            servaddr.sin_port = htons(atoi(envport));
        }
        else {
            ERR_QUIT("Illigal form of  environment RESOFFPORT\n");
        }
    }
    
    if ( (bind (listenfd, (SA *) &servaddr, sizeof (servaddr))) < 0 ) {
        binderrmsg();
        exit(ERROR);
    }

    if ( (listen (listenfd, LISTENQ)) < 0 ) {
        ERR_QUIT("listen error\n");
    }
    
    Signal(SIGCHLD, sig_chld);            /* signal set for child process */

    for ( ; ; ) {
        clilen = sizeof(cliaddr);
        if ( (connfd = accept(listenfd, (SA *) &cliaddr, &clilen)) < 0) {
            if (errno == EINTR)
                continue;
            else
                ERR_QUIT ("accept");
        }
      
        if ( (childpid = fork()) == 0 ) { /* main funct processed in child */
            CHECK (close (listenfd));
            
            if ( (n = read(connfd, recvline, 30)) != 0 ) {
                resoffserve(filename, lockfile, connfd, recvline);
            }
            
            exit(0);
        }
        CHECK (close (connfd)); /* parent closes connected socket */
    }
}
