#include "config.h"
#include "resoff.h"
#include "boolean.h"

/*======================================================================
  Function resoffserve
  Input : file name, lock file name, file descriptor, received string
  Output : return string code to client
  server interpretation of client code and execute corresponding
  functions
======================================================================*/
void resoffserve(char *filename, char *lf, int connfd, char *recvline) 
{
    int fd;                           /* file descriptor */
    int option;                       /* kind of options */
    int argc;                         /* pseudo number of arguments */
    char *argv[5];                    /* pseudo arguments of main function */
    uid_t uid;                        /* user id */
    char buf[3];                      /* sending error message to client */
    int i;

    option = server_cmdchk(recvline, &argc, argv, &uid); /* check command */
    if ( !option ) {                  /* if no proper option */
        snprintf(buf, sizeof(buf), "1\6");
        if ( (write(connfd, buf, strlen(buf))) != strlen(buf) )
            ERR_QUIT ("write in resoffserve");
    }
    else {                            /* got option */
        fd = open (filename, O_RDWR, S_IRUSR | S_IWUSR); /* already created */
        if ( fd < 0 ) {
            openerrmsg();
            remove_lock(lf);
            exit(ERROR);
        }
        switch( option ) {
            case _RESERVE:
                reserve(fd, connfd, argv, uid);
                break;
            case _CANCEL:
                cancel(fd, connfd, argv, uid);
                break;
            case _LIST:
                list(fd, connfd);
                break;
            case _ERR_RESERVE:
            case _ERR_CANCEL:
            case _ERR_SYNTAX:
            case _ERR_CNL_SYNTAX:
                snprintf(buf, sizeof(buf), "5\6");
                if ( (write(connfd, buf, strlen(buf))) != strlen(buf) )
                    ERR_QUIT ("write in syntax error");
                break;
            case _ERR_RSV_SYNTAX:
                snprintf(buf, sizeof(buf), "05\6");
                if ( (write(connfd, buf, strlen(buf))) != strlen(buf) )
                    ERR_QUIT ("write in rsv syntax error");
                break;
            case _ERR_RSV_UID:
                snprintf(buf, sizeof(buf), "02\6");
                if ( (write(connfd, buf, strlen(buf))) != strlen(buf) )
                    ERR_QUIT ("write in rsv uid error");
                break;
            case _ERR_CNL_UID:
                snprintf(buf, sizeof(buf), "2\6");
                if ( (write(connfd, buf, strlen(buf))) != strlen(buf) )
                    ERR_QUIT ("write in cancel uid error");
                break;
            default:
                ERR_QUIT("option");                         /* unreachable */
                break;
        }
        
        close(fd);
        for (i=0; i < argc; i++) {
            free(argv[i]);
        }
    }
}

