/* timecli.c

   An example of using a UDP socket to obtain the current time
   from a remote server, based on RFC 868.

 */

#include "config.h"
#include "timex.h"

int main()
{
    int sockfd;
    struct sockaddr_in servaddr;
    struct hostent *hosts;
    extern char server[25][50];        /* server names */
    extern int action[25];             /* server action */
    extern uint32_t ipaddress[25];    /* 32-bit IPv4 addresses */
    extern struct timeval rectime[25]; /* local time when received */
    struct timeval current;            /* current local time */
    extern struct timeval sendtime;    /* local time when sending */
    socklen_t addrlen;
    uint32_t buf;
    int n;                             /* return value */
    int i, j;                          /* loop variable */
    extern int counter;                /* # of input hosts */
    Sigfunc *sigfunc;
   
    for (i=0; i<25; i++) {
        n = fscanf(stdin, "%s", server[i]);
        if ( n == EOF ) break;
        rectime[i].tv_sec = 0;
    }
   
    counter = i;

    sockfd = socket(AF_INET, SOCK_DGRAM, 0);
    memset (&servaddr, 0, sizeof(servaddr));
    servaddr.sin_family = AF_INET;
    servaddr.sin_port = htons(37);                      /* time server port */

    CHECK ( (n = gettimeofday(&sendtime, NULL)) );      /* get local time */
    
    for (i=0; i<counter; i++) {
        if ( !(hosts = gethostbyname (server[i]))) {
            action[i] = NOFOUND;
        }
        else {
            action[i] = NORMAL;                         /* initial value */
            servaddr.sin_addr = *(struct in_addr *) (hosts->h_addr_list[0]);
            ipaddress[i] = servaddr.sin_addr.s_addr;    /* host's IP address */
            for (j=0; j<6; j++)
                CHECK ( (n = sendto(sockfd, (void *) &buf, 0, 0,
                                    (SA *) &servaddr, sizeof (servaddr))));
        }
    }
    
    addrlen = sizeof (servaddr);
    
    sigfunc = Signal(SIGALRM, timeout_alarm);
    if (alarm(10) != 0) {
        fprintf(stderr, "Alarm was already set\n");
        exit(0);
    }

    while ( (n = recvfrom (sockfd, (void *) &buf, sizeof (buf), 
                           MSG_WAITALL, (SA *) &servaddr, &addrlen)) > 0 ) {
        CHECK ( (n = gettimeofday(&current, NULL)) );
        recordtime(servaddr, current, buf);
    }
        
    alarm(0);
    Signal(SIGALRM, sigfunc);

    print_table(adjust());

    exit (NORMAL);
}
