#include "config.h"
#include "timex.h"

void recordtime
(struct sockaddr_in servaddr, struct timeval current, uint32_t buf)
{
    int index;                         /* index of the host */
    extern struct timeval rectime[25]; /* local time when received */
    extern time_t response[25];        /* response time */
    
    index = getindex(servaddr);

    if ( rectime[index].tv_sec != 0 ) {
        return;
    }
    
    rectime[index] = current;
    response[index] = totime(buf);
    
    return;
}

int getindex(struct sockaddr_in servaddr) 
{
    int i;
    extern int counter;
    extern int action[25];             /* server action */
    extern uint32_t ipaddress[25];    /* 32-bit IPv4 addresses */

    for (i=0; i<counter; i++) {
        if ( action[i] != NOFOUND && ipaddress[i] == servaddr.sin_addr.s_addr )
            return(i);
    }

    fprintf(stderr, "no index for returned server\n");
    exit(ERROR);
}

time_t totime(uint32_t buf)
{
    time_t time;
    uint32_t epoch;
    
    buf = ntohl(buf);
    
    epoch = ((unsigned long) (365) * 70 + 17) * 24 * 60 * 60;
    time = (time_t) (buf - epoch);

    return (time);
}

long adjust()
{
    extern int action[25];             /* server action */
    extern struct timeval rectime[25]; /* local time when received */
    extern time_t response[25];        /* response time */
    extern struct timeval sendtime;    /* local time when sending */
    extern long esttime[25];           /* best-estimated time value */
    int i;                             /* loop variable */
    extern int counter;                /* # of input hosts */
    int number;                        /* number to calculate */
    long sec, usec;                    /* second, micro second */
    double total, deviation, sd;
    long mean;
    
    number = 0;
    total = 0;
    
    for (i=0; i<counter; i++) {
        if ( action[i] != NOFOUND && rectime[i].tv_sec != 0 ) {
            number++;
            sec = rectime[i].tv_sec - sendtime.tv_sec;
            usec = rectime[i].tv_usec - sendtime.tv_usec;
            /* +0.5 is for round */
            sec = (long)(((double)sec + (double)usec / 1000000) / 2 + 0.5);
            esttime[i] = sec;
            total += (double)response[i];
        }
        if ( action[i] != NOFOUND && rectime[i].tv_sec == 0 )
            action[i] = NORESPONSE;
    }
    
    mean = (long) (total / number);
    deviation = 0;
    
    for (i=0; i<counter; i++) {
        if ( action[i] == NORMAL )
            deviation += ((double)response[i] - (double)mean) *
                         ((double)response[i] - (double)mean);
    }
    if ( number > 1 ) deviation = deviation / (number - 1);
    
    sd = sqrt(deviation);
    
    for (i=0; i<counter; i++) {                        /* discard */
        if ( action[i] == NORMAL )
            if ( abs(response[i] - mean) > 2 * sd ) {
                action[i] = DISCARD;
                number--;
                total -= (double)response[i];
            }
    }
    
    mean = (long) (total / number);                    /* new mean */
    
    return (mean);
}

void print_table(long mean)
{
    struct hostent *hosts;
    struct utsname localhost;
    int i;
    int n;
    extern int counter;
    extern char server[25][50];        /* server names */
    extern long esttime[25];           /* best-estimated time value */
    extern struct timeval sendtime;    /* local time when sending */
    struct sockaddr_in localaddr;
    
    CHECK( (n = uname(&localhost)) );
    
    if ( !(hosts = gethostbyname(localhost.nodename)) ) {
        ERR_QUIT("gethostbyname");
    }
    
    localaddr.sin_addr = *(struct in_addr *) (hosts->h_addr_list[0]);
    
    printf("%-20s%-20s%-15s%-14s%-14s\n", "HOST NAME","IP NUMBER","ACTION",
           "EST TIME", "ADJUSTMENT");
    printf("%-20s", "local");
    printf("%-20s", inet_ntoa(localaddr.sin_addr));
    print_action(NORMAL);
    print_esttime(sendtime.tv_sec);
    print_adjust(sendtime.tv_sec - mean);

    for (i=0; i<counter; i++) {
        printf("%-20s", server[i]);
        print_ip(i);
        if ( !(print_action(action[i])) ) {
            print_esttime(response[i] - (time_t)esttime[i]);
            print_adjust((long)response[i] - esttime[i] - mean);
        }
    }
    
}

void print_ip(int i)
{
    extern uint32_t ipaddress[25];    /* 32-bit IPv4 addresses */
    struct in_addr ip_inaddr;          /* a dummny variable */
    
    if ( action[i] == NOFOUND) {
        printf("%-20s", "not found");
        return;
    }
    
    ip_inaddr.s_addr = ipaddress[i];
    
    printf("%-20s", inet_ntoa(ip_inaddr));
}

int print_action(int action)
{
    switch (action) {
        case NORMAL:
            printf("%-15s", "accept");
            return(NORMAL);
            break;
        case NOFOUND:
            printf("%-15s\n", "not found");
            return(SKIP);
            break;
        case NORESPONSE:
            printf("%-15s\n", "no response");
            return(SKIP);
            break;
        case DISCARD:
            printf("%-15s", "discarded");
            return(NORMAL);
            break;
        default:
            fprintf(stderr, "unreachable status in print_action");
            exit(ERROR);
    }
}

void print_esttime(time_t atime)
{
    struct tm *t;

    t = gmtime(&atime);
    
    printf("%02d:%02d:%02d%-6s", t->tm_hour, t->tm_min, t->tm_sec, "");
}

void print_adjust(long sec)
{
    int hour;
    int min;
    
    if ( sec == 0 ) {
        printf(" 00:00:00\n");
        return;
    }
    else if ( sec < 0 )
        printf("+");
    else
        printf("-");
    
    sec = abs(sec);
    
    if ( sec > 60 * 60 ) {
        hour = sec / 3600;
        sec -= (hour * 3600);
        printf("%02d:", hour);
    }
    else
        printf("00:");
    
    if ( sec > 60 ) {
        min = sec / 60;
        sec -= (min * 60);
        printf("%02d:", min);
    }
    else
        printf("00:");
    
    printf ("%02d\n", (int)sec);
}

Sigfunc *
signal(int signo, Sigfunc *func)
{
    struct sigaction  act, oact;
    
    act.sa_handler = func;
    sigemptyset(&act.sa_mask);
    act.sa_flags = 0;
    if (signo == SIGALRM) {
#ifdef  SA_INTERRUPT
        act.sa_flags |= SA_INTERRUPT;   /* SunOS 4.x */
#endif
    } else {
#ifdef  SA_RESTART
        act.sa_flags |= SA_RESTART;     /* SVR4, 44BSD */
#endif
    }
    if (sigaction(signo, &act, &oact) < 0)
        return(SIG_ERR);
    return(oact.sa_handler);
}

Sigfunc *
Signal(int signo, Sigfunc *func)        /* for our signal() function */
{
    Sigfunc *sigfunc;
    
    if ( (sigfunc = signal(signo, func)) == SIG_ERR) {
        fprintf(stderr, "signal error");
        exit(0);
    }
    
    return(sigfunc);
}

void
timeout_alarm(int signo)
{
    return;
}
