#! /bin/sh -f

binname=$1

unknown () {
    echo "Unknown system type: $unamestr"
    echo "Please report this problem to us."
    exit 1
}

run () {
    unset TWINDIR
    if [ "x$HNCDIR" = "x" ] ; then
        HNCDIR=/usr/hwpx; export HNCDIR
    fi
    binfile="$HNCDIR/arch/$archname/`basename $0`"
    if [ ! -f $binfile ] ; then
        echo "Execution file '$binfile' is not found."
	echo "$0 is not correctly installed."
	exit 1
    fi
    exec $binfile $@
}

#
# for each platform
#
run_solaris () {
    archname="solaris"
    run $@
}

run_irix () {
    archname="irix"
    run $@
}

run_linux () {
    archname="linux"
    run $@
}

run_aix () {
    archname="aix"
    run $@
}

run_sco () {
    archname="sco"
    run $@
}

run_hpux () {
    archname="hpux"
    run $@
}

#

unamestr=`uname -a`

case $unamestr in
    SunOS*)     run_solaris $@ ;;
    AIX*)       run_aix $@ ;;
    IRIX*)      run_irix $@ ;;
    Linux*)     run_linux $@ ;;
    SCO*)       run_sco $@ ;;
    HP-UX*)     run_hpux $@ ;;
    *)          unknown ;;
esac
