#include "xresoff.h"

/*======================================================================
  Function option_menu
  Input : GtkWidget w and pointer data (parameter for options menu)
  Output : 
  make a selection for the host or port number
======================================================================*/
void option_menu ( GtkWidget *w, gpointer data )
{
    
    switch ( GPOINTER_TO_INT(data) ) {
        case 1:
            xsethost();
            break;
        case 2:
            xsetport();
            break;
        default:
            break;
    }
    
}

/*======================================================================
  Function xresoff_menu
  Input : GtkWidget w and pointer data (parameter for resoff menu)
  Output : menu selection message
  make a selection for the resoff commands - reserve, cancel, and list
======================================================================*/
void xresoff_menu( GtkWidget *w, gpointer data )
{
    GtkLabel *a_label;
    a_label = GTK_LABEL(m_label);

    switch ( GPOINTER_TO_INT(data) ) {
        case 1:
            gtk_label_set_text(a_label, "Reserve!");
            xreserve();
            break;
        case 2:
            gtk_label_set_text(a_label, "Cancel!");
            xcancel();
            break;
        case 3:
            gtk_label_set_text(a_label, "List!");
            xlist();
            break;
        default:
            break;
    }
    
}

/*======================================================================
  Function get_entry_day
  Input : GtkWidget w and pointer data (input character string day)
  Output : print on arg_day variable
  copy the input day string to arg_day variable
======================================================================*/
void get_entry_day ( GtkWidget *widget,
                      gpointer data)
{
    sprintf(arg_day, "%s", (char *)gtk_entry_get_text( (GtkEntry *) data ));
}

/*======================================================================
  Function get_entry_time
  Input : GtkWidget w and pointer data (input character string time)
  Output : print on arg_time variable
  copy the input time string to arg_time variable
======================================================================*/
void get_entry_time ( GtkWidget *widget,
                      gpointer data)
{
    sprintf(arg_time, "%s", (char *)gtk_entry_get_text( (GtkEntry *) data ));
}

/*======================================================================
  Function get_entry_course
  Input : GtkWidget w and pointer data (input character string course)
  Output : print on arg_course variable
  copy the input course string to arg_course variable
======================================================================*/
void get_entry_course ( GtkWidget *widget,
                      gpointer data)
{
    sprintf(arg_course, "%s", (char *)gtk_entry_get_text( (GtkEntry *) data ));
}

/*======================================================================
  Function get_entry_id
  Input : GtkWidget w and pointer data (input character string id)
  Output : print on arg_id variable
  copy the input id string to arg_id variable
======================================================================*/
void get_entry_id ( GtkWidget *widget,
                      gpointer data)
{
    sprintf(arg_id, "%s", (char *)gtk_entry_get_text( (GtkEntry *) data ));
}

/*======================================================================
  Function get_entry_host
  Input : GtkWidget w and pointer data (input character string host)
  Output : print on arg_host variable
  copy the input host string to arg_host variable
======================================================================*/
void get_entry_host ( GtkWidget *widget,
                      gpointer data)
{
    sprintf(server, "%s", (char *)gtk_entry_get_text( (GtkEntry *) data));
}

/*======================================================================
  Function get_entry_port
  Input : GtkWidget w and pointer data (input character string port)
  Output : print on arg_port variable
  copy the input port string to arg_port variable
======================================================================*/
void get_entry_port ( GtkWidget *widget,
                      gpointer data)
{
    sprintf(port, "%s", (char *)gtk_entry_get_text( (GtkEntry *) data));
}

/*======================================================================
  Function get_env
  Input : environment pathname
  Output : get environment for the program execution
  get environment path of the RESOFFBINDER and set the path name of
  resoffc program
======================================================================*/
void get_env () 
{
    char *envpath;
    
    envpath = getenv("RESOFFBINDER");
    if ( envpath == NULL ) {
        fprintf(stderr, "RESOFFBINDER environment needed\n");
        exit(-1);
    }
    sprintf(pathname, envpath);
    if ( pathname[strlen(envpath) - 1] != '/' )
        strcat(pathname, "/");
    strcat(pathname, "resoffc");
}

/*======================================================================
  Function xsethost
  Input : 
  Output : input window for the hostname of resoffd server
  make an input window for the hostname of resoffd server
======================================================================*/
void xsethost()
{
    GtkWidget *window, *label, *button;
    GtkWidget *entry_host;
    GtkWidget *vbox;
    
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    label = gtk_label_new("Server name:");
    gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 1);
    
    entry_host = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(entry_host), "mom.cs.fsu.edu");
    gtk_box_pack_start (GTK_BOX (vbox), entry_host, FALSE, FALSE, 1);
   
    button = gtk_button_new_with_label ("Get");
    gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 1);
    gtk_signal_connect (GTK_OBJECT(button), "clicked",
                        GTK_SIGNAL_FUNC(get_entry_host),
                        (gpointer)entry_host);

    gtk_signal_connect (GTK_OBJECT(button), "clicked",
                        GTK_SIGNAL_FUNC(set_hostname), NULL);
    
    gtk_signal_connect_object (GTK_OBJECT(button), "clicked",
                               GTK_SIGNAL_FUNC(gtk_widget_destroy),
                               GTK_OBJECT(window));

    gtk_widget_show_all(window);

}

/*======================================================================
  Function get_hostname
  Input : GtkWidget w and pointer data (server name)
  Output : message to the main window
  show changed server name and message to the main window
======================================================================*/
void set_hostname ( GtkWidget *widget, gpointer data )
{
    GtkLabel *a_label;
    
    setenv("RESOFFHOST", server, 1);
    a_label = GTK_LABEL(s_label);
    gtk_label_set_text(a_label, (gchar *) server);
    gtk_label_set_text(GTK_LABEL(m_label),
                       "Server hostname has been changed.");
}

/*======================================================================
  Function xsetport
  Input : 
  Output : input window for the port number for resoffc
  make an input window for the port number for resoffc
======================================================================*/
void xsetport()
{
    GtkWidget *window, *label, *button;
    GtkWidget *entry_port;
    GtkWidget *vbox;
    
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    label = gtk_label_new("Port Number:");
    gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 1);
    
    entry_port = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(entry_port), "11429");
    gtk_box_pack_start (GTK_BOX (vbox), entry_port, FALSE, FALSE, 1);
   
    button = gtk_button_new_with_label ("Get");
    gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 1);
    gtk_signal_connect (GTK_OBJECT(button), "clicked",
                        GTK_SIGNAL_FUNC(get_entry_port),
                        (gpointer)entry_port);

    gtk_signal_connect (GTK_OBJECT(button), "clicked",
                        GTK_SIGNAL_FUNC(set_portnumber), NULL);
    
    gtk_signal_connect_object (GTK_OBJECT(button), "clicked",
                               GTK_SIGNAL_FUNC(gtk_widget_destroy),
                               GTK_OBJECT(window));

    gtk_widget_show_all(window);

}

/*======================================================================
  Function set_portnumber
  Input : GtkWidget w and pointer data (port number)
  Output : message to the main window
  show changed port number and message to the main window
======================================================================*/
void set_portnumber ( GtkWidget *widget, gpointer data )
{
    GtkLabel *a_label;
    
    setenv("RESOFFPORT", port, 1);
    a_label = GTK_LABEL(p_label);
    gtk_label_set_text(a_label, (gchar *) port);
    gtk_label_set_text(GTK_LABEL(m_label), "Port number has been changed.");
}

